# -*- coding: utf-8 -*-

from imio.events.core.utils import reload_faceted_config
from imio.smartweb.common.upgrades import upgrades
from plone import api
from zope.globalrequest import getRequest

import logging

logger = logging.getLogger("imio.events.core")


def refresh_objects_faceted(context):
    request = getRequest()
    brains = api.content.find(portal_type=["imio.events.Entity", "imio.events.Agenda"])
    for brain in brains:
        obj = brain.getObject()
        reload_faceted_config(obj, request)
        logger.info("Faceted refreshed on {}".format(obj.Title()))


def add_event_dates_index(context):
    catalog = api.portal.get_tool("portal_catalog")
    catalog.addIndex("event_dates", "KeywordIndex")
    catalog.manage_reindexIndex(ids=["event_dates"])
    logger.info("Added and indexed event_dates KeywordIndex")


def reindex_searchable_text(context):
    upgrades.reindex_searchable_text(context)


def add_translations_indexes(context):
    catalog = api.portal.get_tool("portal_catalog")

    new_indexes = ["translated_in_nl", "translated_in_de", "translated_in_en"]
    indexes = catalog.indexes()
    indexables = []
    for new_index in new_indexes:
        if new_index in indexes:
            continue
        catalog.addIndex(new_index, "BooleanIndex")
        indexables.append(new_index)
        logger.info(f"Added BooleanIndex for field {new_index}")
    if len(indexables) > 0:
        logger.info(f"Indexing new indexes {', '.join(indexables)}")
        catalog.manage_reindexIndex(ids=indexables)

    new_metadatas = ["title_fr", "title_nl", "title_de", "title_en"]
    metadatas = list(catalog.schema())
    must_reindex = False
    for new_metadata in new_metadatas:
        if new_metadata in metadatas:
            continue
        catalog.addColumn(new_metadata)
        must_reindex = True
        logger.info(f"Added {new_metadata} metadata")
    if must_reindex:
        logger.info("Reindexing catalog for new metadatas")
        catalog.clearFindAndRebuild()


def reindex_catalog(context):
    catalog = api.portal.get_tool("portal_catalog")
    catalog.clearFindAndRebuild()


def remove_searchabletext_fr(context):
    catalog = api.portal.get_tool("portal_catalog")
    catalog.manage_delIndex("SearchableText_fr")


def remove_title_description_fr(context):
    catalog = api.portal.get_tool("portal_catalog")
    catalog.delColumn("title_fr")
    catalog.delColumn("description_fr")


def reindex_event_dates_index(context):
    catalog = api.portal.get_tool("portal_catalog")
    catalog.manage_reindexIndex(ids=["event_dates"])
    logger.info("Reindexed event_dates index")


def add_dates_indexes(context):
    catalog = api.portal.get_tool("portal_catalog")

    new_indexes = ["first_start", "first_end"]
    indexes = catalog.indexes()
    indexables = []
    for new_index in new_indexes:
        if new_index in indexes:
            continue
        catalog.addIndex(new_index, "FieldIndex")
        indexables.append(new_index)
        logger.info(f"Added FieldIndex for field {new_index}")
    if len(indexables) > 0:
        logger.info(f"Indexing new indexes {', '.join(indexables)}")
        catalog.manage_reindexIndex(ids=indexables)

    new_metadatas = ["first_start", "first_end"]
    metadatas = list(catalog.schema())
    must_reindex = False
    for new_metadata in new_metadatas:
        if new_metadata in metadatas:
            continue
        catalog.addColumn(new_metadata)
        must_reindex = True
        logger.info(f"Added {new_metadata} metadata")
    if must_reindex:
        logger.info("Reindexing catalog for new metadatas")
        catalog.clearFindAndRebuild()



# -*- coding: utf-8 -*-

from datetime import date
from imio.events.core.utils import expand_occurences
from imio.events.core.utils import get_start_date
from imio.smartweb.common.utils import is_log_active
from plone import api
from plone.restapi.batching import HypermediaBatch
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.search.handler import SearchHandler
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.services import Service
from zope.component import getMultiAdapter

import logging
import time

logger = logging.getLogger("imio.events.core")


class EventsEndpointGet(Service):
    def reply(self):
        query = self.request.form.copy()
        query = unflatten_dotted_dict(query)
        return EventsEndpointHandler(self.context, self.request).search(query)


class EventsEndpointHandler(SearchHandler):
    """ """

    # we receive b_size and b_start from smartweb with values already set
    # So we ignore these values but we must stock these to use it ...
    ignored_params = ["b_size", "b_start"]

    def search(self, query=None):
        if query is None:
            query = {}
        b_size = query.get("b_size") or 20
        b_start = query.get("b_start") or 0

        for param in self.ignored_params:
            if param in query:
                del query[param]

        if "fullobjects" in query:
            fullobjects = True
            del query["fullobjects"]
        else:
            fullobjects = False

        query["portal_type"] = "imio.events.Event"
        query["review_state"] = "published"
        query["b_size"] = 10000

        def cascading_agendas(initial_agenda):
            global_list = []

            def recursive_generator(agenda_UID):
                nonlocal global_list
                obj = api.content.get(UID=agenda_UID)
                populating_agendas = []
                for rv in obj.populating_agendas:
                    if hasattr(rv, "to_object"):
                        if (
                            rv.to_object is not None
                            and rv.to_object.UID() not in global_list
                        ):
                            obj = rv.to_object
                            status = api.content.get_state(obj)
                            if status == "published":
                                # to cover initial_agenda UID
                                populating_agendas.append(agenda_UID)
                                global_list.append(agenda_UID)

                                # to cover RelationValue agenda UID
                                populating_agendas.append(rv.to_object.UID())
                                global_list.append(rv.to_object.UID())
                for agenda_UID in populating_agendas:
                    yield from recursive_generator(agenda_UID)
                yield agenda_UID

            yield from recursive_generator(initial_agenda)

        # To cover cascading "populating_agendas" field
        if "selected_agendas" in query:
            selected_agendas = [
                agenda_UID
                for agenda_UID in cascading_agendas(query["selected_agendas"])
            ]
            all_agendas = list(set(selected_agendas))
            query["selected_agendas"] = all_agendas

        # Only future events
        today = date.today().isoformat()
        query["event_dates"] = {"query": today, "range": "min"}
        tps1 = time.time()
        self._constrain_query_by_path(query)
        tps2 = time.time()
        query = self._parse_query(query)
        tps3 = time.time()
        lazy_resultset = self.catalog.searchResults(**query)
        tps4 = time.time()
        if "metadata_fields" not in self.request.form:
            self.request.form["metadata_fields"] = []
        self.request.form["metadata_fields"] += [
            "recurrence",
            "whole_day",
            "first_start",
            "first_end",
            "open_end",
        ]
        # ISerializeToJson use a default request batch so we force a "full" b_size and a "zero" b_start
        self.request.form["b_size"] = 10000
        self.request.form["b_start"] = 0
        results = getMultiAdapter((lazy_resultset, self.request), ISerializeToJson)(
            fullobjects=fullobjects
        )
        tps5 = time.time()
        expanded_occurences = expand_occurences(results.get("items"))
        sorted_expanded_occurences = sorted(expanded_occurences, key=get_start_date)
        tps6 = time.time()

        # It's time to get real b_size/b_start from the smartweb query
        self.request.form["b_size"] = b_size
        self.request.form["b_start"] = b_start
        batch = HypermediaBatch(self.request, sorted_expanded_occurences)
        tps7 = time.time()
        if is_log_active():
            logger.info(f"query : {results['@id']}")
            logger.info(f"time constrain_query_by_path : {tps2 - tps1}")
            logger.info(f"time _parse_query : {tps3 - tps2}")
            logger.info(f"time catalog lazy_resultset : {tps4 - tps3}")
            logger.info(f"time MultiAdapter fullobj : {tps5 - tps4}")
            logger.info(f"time occurences : {tps6 - tps5}")
            logger.info(f"time batch : {tps7 - tps6}")
            logger.info(f"time (total) : {tps7 - tps1}")

        results = {}
        results["@id"] = batch.canonical_url
        results["items_total"] = batch.items_total
        links = batch.links
        if links:
            results["batching"] = links
        results["items"] = [event for event in batch]
        return results



# -*- coding: utf-8 -*-

from imio.smartweb.common.config import VOCABULARIES_MAPPING

VOCABULARIES_MAPPING.update(
    {
        "category": "imio.events.vocabulary.EventsCategories",
        "local_category": "imio.events.vocabulary.EventsLocalCategories",
    }
)



# -*- coding: utf-8 -*-

import os


def make_named_image(filename="plone.png"):
    path = os.path.join(os.path.dirname(__file__), f"resources/{filename}")
    with open(path, "rb") as f:
        image_data = f.read()
    return {"filename": filename, "data": image_data}



# -*- coding: utf-8 -*-

from datetime import datetime
from datetime import timezone
from freezegun import freeze_time
from imio.events.core.testing import IMIO_EVENTS_CORE_INTEGRATION_TESTING
from imio.events.core.utils import expand_occurences
from imio.events.core.utils import get_agenda_for_event
from imio.events.core.utils import get_agendas_uids_for_faceted
from imio.events.core.utils import get_entity_for_obj
from imio.events.core.utils import get_start_date
from imio.events.core.utils import remove_zero_interval_from_recrule
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class TestAgenda(unittest.TestCase):
    layer = IMIO_EVENTS_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.authorized_types_in_agenda = [
            "imio.events.Folder",
            "imio.events.Event",
        ]
        self.unauthorized_types_in_agenda = [
            "imio.events.Agenda",
            "Document",
            "File",
            "Image",
        ]

        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.parent = self.portal
        self.entity1 = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            id="entity1",
        )
        self.agenda1 = api.content.create(
            container=self.entity1,
            type="imio.events.Agenda",
            id="agenda1",
        )
        self.event1 = api.content.create(
            container=self.agenda1,
            type="imio.events.Event",
            id="event1",
        )
        self.entity2 = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            id="entity2",
        )
        self.agenda2 = api.content.create(
            container=self.entity2,
            type="imio.events.Agenda",
            id="agenda2",
        )
        self.event2 = api.content.create(
            container=self.agenda2,
            type="imio.events.Event",
            id="event2",
        )
        self.agenda3 = api.content.create(
            container=self.entity1,
            type="imio.events.Agenda",
            id="agenda3",
        )

    def test_get_entity_for_obj(self):
        self.assertEqual(get_entity_for_obj(self.entity1), self.entity1)
        self.assertEqual(get_entity_for_obj(self.agenda1), self.entity1)
        self.assertEqual(get_entity_for_obj(self.event1), self.entity1)

    def test_get_agenda_for_event(self):
        self.assertEqual(get_agenda_for_event(self.event1), self.agenda1)
        self.assertEqual(get_agenda_for_event(self.event2), self.agenda2)

    def test_get_agendas_uids_for_faceted(self):
        with self.assertRaises(NotImplementedError):
            get_agendas_uids_for_faceted(self.event1)
        self.assertEqual(
            get_agendas_uids_for_faceted(self.agenda1), [self.agenda1.UID()]
        )
        default_agendas = self.entity1.listFolderContents(
            contentFilter={"portal_type": "imio.events.Agenda"}
        )
        uids = []
        for event in default_agendas:
            uids.append(event.UID())
        self.assertEqual(
            get_agendas_uids_for_faceted(self.entity1),
            uids,
        )
        self.assertIn(self.agenda1.UID(), get_agendas_uids_for_faceted(self.entity1))
        self.assertIn(self.agenda3.UID(), get_agendas_uids_for_faceted(self.entity1))

    @freeze_time("2022-11-10")
    def test_expand_occurences(self):
        # test without occurence
        events = [
            {
                "start": "2022-11-13T12:00:00+00:00",
                "end": "2022-11-13T13:00:00+00:00",
                "first_start": "2022-11-13T12:00:00+00:00",
                "first_end": "2022-11-13T13:00:00+00:00",
                "recurrence": None,
                "open_end": False,
                "whole_day": False,
            }
        ]
        expanded_events = expand_occurences(events)
        self.assertEqual(len(expanded_events), 1)
        events = [
            {
                "start": "2022-11-13T12:00:00+00:00",
                "end": "2022-11-14T13:00:00+00:00",
                "first_start": "2022-11-13T12:00:00+00:00",
                "first_end": "2022-11-14T13:00:00+00:00",
                "recurrence": None,
                "open_end": False,
                "whole_day": False,
            }
        ]
        expanded_events = expand_occurences(events)
        self.assertEqual(len(expanded_events), 1)

        # test range start for occurences
        events = [
            {
                "start": "2022-11-15T12:00:00+00:00",
                "end": "2022-11-15T13:00:00+00:00",
                "first_start": "2022-11-01T12:00:00+00:00",
                "first_end": "2022-11-01T13:00:00+00:00",
                "recurrence": "RRULE:FREQ=WEEKLY;COUNT=5",
                "open_end": False,
                "whole_day": False,
            }
        ]
        expanded_events = expand_occurences(events)
        self.assertEqual(len(expanded_events), 3)

        # test occurences data
        events = [
            {
                "start": "2022-11-13T12:00:00+00:00",
                "end": "2022-11-13T13:00:00+00:00",
                "first_start": "2022-11-13T12:00:00+00:00",
                "first_end": "2022-11-13T13:00:00+00:00",
                "recurrence": "RRULE:FREQ=WEEKLY;COUNT=5",
                "open_end": False,
                "whole_day": False,
            }
        ]
        expanded_events = expand_occurences(events)
        self.assertEqual(len(expanded_events), 5)
        self.assertEqual(expanded_events[-1]["start"], "2022-12-11T12:00:00+00:00")
        self.assertEqual(expanded_events[-1]["end"], "2022-12-11T13:00:00+00:00")
        events = [
            {
                "start": "2022-11-13T12:00:00+00:00",
                "end": "2022-11-13T12:00:00+00:00",
                "first_start": "2022-11-13T12:00:00+00:00",
                "first_end": "2022-11-13T12:00:00+00:00",
                "recurrence": "RRULE:FREQ=WEEKLY;COUNT=5",
                "open_end": False,
                "whole_day": True,
            }
        ]
        expanded_events = expand_occurences(events)
        self.assertEqual(expanded_events[-1]["start"], "2022-12-11T12:00:00+00:00")
        self.assertEqual(expanded_events[-1]["end"], "2022-12-12T11:59:59+00:00")
        events = [
            {
                "start": "2022-11-13T00:00:00+00:00",
                "end": "2022-11-13T23:59:59+00:00",
                "first_start": "2022-11-13T00:00:00+00:00",
                "first_end": "2022-11-13T23:59:59+00:00",
                "recurrence": "RRULE:FREQ=WEEKLY;COUNT=5",
                "open_end": True,
                "whole_day": True,
            }
        ]
        expanded_events = expand_occurences(events)
        self.assertEqual(expanded_events[-1]["start"], "2022-12-11T00:00:00+00:00")
        self.assertEqual(expanded_events[-1]["end"], "2022-12-11T23:59:59+00:00")

    def test_get_start_date(self):
        event = {
            "start": "2022-11-13T12:00:00+00:00",
            "end": "2022-11-13T13:00:00+00:00",
            "recurrence": None,
            "open_end": False,
            "whole_day": False,
        }
        start_date = get_start_date(event)
        result = datetime(2022, 11, 13, 12, 0, tzinfo=timezone.utc)
        self.assertEqual(start_date, result)

    def test_remove_zero_interval_from_recrule(self):
        recrule = "RRULE:FREQ=WEEKLY;COUNT=5"
        self.assertEqual(
            remove_zero_interval_from_recrule(recrule), "RRULE:FREQ=WEEKLY;COUNT=5"
        )
        recrule = "RRULE:FREQ=WEEKLY;INTERVAL=1"
        self.assertEqual(
            remove_zero_interval_from_recrule(recrule), "RRULE:FREQ=WEEKLY;INTERVAL=1"
        )
        recrule = "RRULE:FREQ=WEEKLY;INTERVAL=1;COUNT=5"
        self.assertEqual(
            remove_zero_interval_from_recrule(recrule),
            "RRULE:FREQ=WEEKLY;INTERVAL=1;COUNT=5",
        )
        recrule = "RRULE:FREQ=WEEKLY;INTERVAL=0"
        self.assertEqual(
            remove_zero_interval_from_recrule(recrule), "RRULE:FREQ=WEEKLY"
        )
        recrule = "RRULE:FREQ=WEEKLY;INTERVAL=0;COUNT=5"
        self.assertEqual(
            remove_zero_interval_from_recrule(recrule), "RRULE:FREQ=WEEKLY;COUNT=5"
        )
        recrule = "RRULE:FREQ=WEEKLY;INTERVAL=0\nRDATE:2023-09-12T000000"
        self.assertEqual(
            remove_zero_interval_from_recrule(recrule),
            "RRULE:FREQ=WEEKLY\nRDATE:2023-09-12T000000",
        )
